@@ -1,7 +1,19 @@
 #!/usr/bin/env python3
 import re
+import sys
+import codecs
 import random
 
+__all__ = ['Markov', 'mixed_encoding_extracting', 'REGEXPS']
+
+try:
+    import chardet
+except ImportError:
+    try:
+        import charade as chardet
+    except ImportError:
+        chardet = None
+
 class Markov(object):
 
     def __init__(self, messages=None):
@@ -85,7 +97,24 @@ class Markov(object):
             gen_words = reversed(gen_words)
         return ' '.join(filter(lambda x:x!='\n', gen_words))
 
+def mixed_encoding_extracting(f):
+    def newf(msg):
+        encoding = 'utf8'
+        if chardet:
+            encoding = chardet.detect(msg)['encoding']
+        try:
+            msg = msg.decode(encoding)
+        except UnicodeDecodeError:
+            return None
+        else:
+            return f(msg)
+    return newf
 
+REGEXPS = {
+    'weechat': '^.*\t.+\t(<[^ ]+> )?(?P<message>.*)$',
+    'xchat': '[a-z.]+ [0-9]+ [0-9:]+ <[^ ]+> (<[^ ]+> )?(?P<message>.*)$',
+    'supybot': '^[^ ]*  (<[^ ]+> )?(?P<message>.*)$',
+}
 
 
 def main():
@@ -95,11 +124,15 @@ def main():
     if sys.argv[1] not in REGEXPS:
         print('Supported extracters: %s' % ', '.join(REGEXPS))
         exit(1)
-    regexp = REGEXPS[sys.argv[1]]
-    extracter = re.compile(regexp)
+    regexp = re.compile(REGEXPS[sys.argv[1]])
+    @mixed_encoding_extracting
+    def extracter(x):
+        msg = regexp.match(x)
+        if msg:
+            return msg.group('message')
     m = Markov()
     for filename in sys.argv[2:]:
-        with open(filename) as fd:
+        with codecs.open(filename, 'rb') as fd:
             m.feed_from_file(fd, extracter)
 
     for x in range(0, 500):
